import { TemplateObject } from "logimat";
/**
 * Creates a new behavior.
 * Usage: createBehavior!(name: string, body: Body);
 */
export declare const createBehavior: TemplateObject;
/**
 * Add to an already existing behavior.
 * Usage: extendBehavior!(name: string, body: Body);
 */
export declare const extendBehavior: TemplateObject;
/**
 * Marks a behavior's variable as mutable (changable after compilation).
 * Usage: setMut!(variableName: string);
 */
export declare const setMut: TemplateObject;
/**
 * Marks a behavior's variable as mutable (changable after compilation), but inlines it instead of exporting it.
 * Usage: setInline!(variableName: string);
 */
export declare const setInline: TemplateObject;
/**
 * Get the value of a behavior's variable. A boolean can be supplied to get the currently saved value instead of the current value.
 * Usage: getVal!(variableName: string, saved?: boolean);
 */
export declare const getVal: TemplateObject;
/**
 * Set the value of a behavior's variable (during compilation). This must be used before a variable is marked as mutable.
 * Usage: setVal!(variableName: string, val: number);
 */
export declare const setVal: TemplateObject;
/**
 * Set the value of a behavior's variable (during compilation) using a zero-based argument that was passed to the behavior when it was added to the GameObject. This must be used before a variable is marked as mutable.
 * Usage: setValArgs!(variableName: string, arg: number, defaultVal?: number);
 */
export declare const setValArgs: TemplateObject;
/**
 * Set the value of a behavior's variable on update (during runtime). This must be used after a variable is marked as mutable.
 * Usage: setValAction!(variableName: string, body: Body, priority?: number = 0, exported?: boolean = false, variable?: boolean = false);
 */
export declare const setValAction: TemplateObject;
/**
 * Create an action that sets the value of a behavior's variable (during runtime). This must be run manually (or as the click property of an object). This must be used after a variable is marked as mutable.
 * Usage: noRegisterSetValAction!(variableName: string, body: Body, actionName?: string, priority?: number = 0);
 */
export declare const noRegisterSetValAction: TemplateObject;
/**
 * Create custom declarations on the behavior. This is ideal for graphs, polygons, points, etc, but should not be used for named declarations ((functions, consts, etc).
 * Usage: behaviorCustom!(body: Body, priority?: number = 0);
 */
export declare const behaviorCustom: TemplateObject;
/**
 * Allows modifying the display properties of any object that this is attached to.
 * Usage: setDisplay!(body: Body, priority?: number = 0);
 */
export declare const setDisplay: TemplateObject;
/**
 * Gets a zero-based argument that was passed to the behavior when it was added to the GameObject.
 * Usage: getBehaviorArgs!(arg: number);
 */
export declare const getBehaviorArgs: TemplateObject;
/**
 * Allows creating helper methods at certain priorities, and which are only output if the behavior they are attached to is used.
 * Usage: helper!(body: Body, priority?: number = 0);
 */
export declare const helper: TemplateObject;
/**
 * Gets the ID of the object that this behavior is currently attached to.
 * Usage: objectID!();
 */
export declare const objectID: TemplateObject;
/**
 * For internal use only.
 */
export declare const setBehavior: TemplateObject;
