"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.finalize2 = exports.finalize1 = exports.finalize = exports.registerAction = exports.initialize = void 0;
const util_1 = require("../util");
const main_1 = __importDefault(require("../components/main"));
/**
 * Initialize the game. This must be the first template called.
 */
exports.initialize = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        return main_1.default;
    }
};
/**
 * Registers an action and runs it on update.
 * Usage: registerAction!(actionName: string);
 */
exports.registerAction = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const actionName = (0, util_1.getString)(args, state, 0, "An action name is required!");
        state.graphgame.finalActions.push(actionName);
        return "";
    }
};
/**
 * For internal use only.
 */
exports.finalize = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const output = [];
        for (const action of state.graphgame.selects) {
            output.push(action(state));
        }
        return output.join("\n") + "\nfinalize1!();";
    }
};
/**
 * For internal use only.
 */
exports.finalize1 = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        state.graphgame.postInit = true;
        const output = [];
        const priorityOutput = {};
        for (const objectID in state.graphgame.objects) {
            for (const priority of Object.keys(state.graphgame.objects[objectID].behaviorPostActions)) {
                if (!priorityOutput.hasOwnProperty(priority))
                    priorityOutput[priority] = [];
                priorityOutput[priority].push(...state.graphgame.objects[objectID].behaviorPostActions[priority]);
            }
        }
        //Sort in ascending order.
        for (const priority of Object.keys(priorityOutput).map(priority => parseInt(priority)).sort((a, b) => a - b)) {
            output.push(...priorityOutput[priority]);
        }
        for (const action of state.graphgame.postActions) {
            output.push(action(state));
        }
        return output.join("\n") + "\nfinalize2!();";
    }
};
/**
 * For internal use only.
 */
exports.finalize2 = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const output = [];
        for (const name in state.graphgame.actions) {
            const val = state.graphgame.actions[name];
            output.push(`action ${name + "u"} = ${name} {
                state = ${val};
            }`);
            state.graphgame.finalActions.push(name + "u");
        }
        if (state.graphgame.finalActions.length > 0)
            output.push("actions m_ain = " + state.graphgame.finalActions.join(", ") + ";");
        return output.join("\n");
    }
};
//# sourceMappingURL=main.js.map