"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setObject = exports.setObjectVal = exports.useBehaviorPost = exports.useBehavior = exports.createObject = void 0;
const GameObject_1 = require("../types/GameObject");
const SemiMutable_1 = require("../types/SemiMutable");
const util_1 = require("../util");
/**
 * Creates a new object.
 * Usage: createObject!(body: Body);
 */
exports.createObject = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        //Store the next ID, then increment it.
        const id = state.graphgame.nextObjectId++;
        const body = (0, util_1.getBlock)(args, state, 0, "An object definition is required!");
        state.graphgame.objects[id] = new GameObject_1.GameObject(id);
        state.graphgame.currentObject = id;
        return `useBehavior!("transform");${body}setObject!();`;
    }
};
/**
 * Add a behavior to an object.
 * Usage: useBehavior!(behaviorName: string);
 */
exports.useBehavior = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const id = state.graphgame.currentObject;
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        (0, util_1.objectCheck)(state, id);
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].compilePost(id, state.graphgame.objects[id].behaviorPostActions, `useBehaviorPost!(${id}, \"${name}\"${args.length > 2 ? ", " + args.slice(1).map(val => {
            if (typeof (val) === "string")
                return `"${val}"`;
            else if (typeof (val) === "object" && val["block"])
                return "{" + val["value"] + "}";
            else
                return val.toString();
        }).join(", ") : ""});`);
        state.graphgame.behaviors[name].compileDisplay(state.graphgame.objects[id].displayProperties);
        state.graphgame.objects[id].behaviors.push(name);
        state.graphgame.lastObjectBehaviorId = id;
        state.graphgame.lastObjectBehaviorArgs = args.slice(1);
        return state.graphgame.behaviors[name].compile(id);
    }
};
/**
 * Internal use only.
 */
exports.useBehaviorPost = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const id = (0, util_1.getNum)(args, state, 0, "An object ID is required!");
        const name = (0, util_1.getString)(args, state, 1, "A behavior name is required!").trim().toLowerCase();
        (0, util_1.objectCheck)(state, id);
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.lastObjectBehaviorId = id;
        state.graphgame.lastObjectBehaviorArgs = args.slice(2);
        return "";
    }
};
/**
 * Set the value of an object's variable (during compilation). This must be used before a variable is marked as mutable.
 * Usage: setObjectVal!(variableName: string, val: number);
 */
exports.setObjectVal = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const id = state.graphgame.currentObject;
        const name = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase().replace(/[._]/g, "");
        const val = (0, util_1.getNum)(args, state, 1, "A value is required!");
        (0, util_1.objectCheck)(state, id);
        if (!state.graphgame.objects[id].hasOwnProperty(name)) {
            state.graphgame.objects[id][name] = new SemiMutable_1.SemiMutable(name, id, val, state);
        }
        else {
            //The error message is included in here.
            (0, util_1.getSemiMut)(state, id, name).set(val);
        }
        return "";
    }
};
/**
 * For internal use only.
 */
exports.setObject = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerInnerCheck)(context);
        state.graphgame.currentObject = (0, util_1.getNum)(args, state, 0) || 0;
        return "";
    }
};
//# sourceMappingURL=object.js.map