"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Behavior = void 0;
class Behavior {
    constructor(name) {
        this.parts = [];
        this.postParts = {};
        this.helpers = {};
        this.displays = {};
        this.nextId = 1;
        this.name = name;
        //Increment and store the id for each gameobject that the behavior is added to.
        this.parts.push((id) => `selectID!(${id});
        setValSelect!("${name}.id", ${this.nextId++});
        selectID!();`);
    }
    /**
     * Add a new part to the behavior.
     */
    add(val) {
        this.parts.push(val);
    }
    /**
     * Add a new post part to the behavior.
     */
    addPost(val, priority) {
        if (!this.postParts.hasOwnProperty(priority))
            this.postParts[priority] = [];
        this.postParts[priority].push(val);
    }
    /**
     * Adds a helper method/variable to this behavior.
     * @param val {string} - is the helper.
     * @param priority {number} - is the priority of the helper.
     */
    addHelper(val, priority) {
        if (!this.helpers.hasOwnProperty(priority))
            this.helpers[priority] = [];
        this.helpers[priority].push(val);
    }
    /**
     * Adds a display property to this behavior.
     * @param val {string} - is the display property.
     * @param priority {number} - is the priority of the display property.
     */
    addDisplay(val, priority) {
        if (!this.displays.hasOwnProperty(priority))
            this.displays[priority] = [];
        this.displays[priority].push(val);
    }
    /**
     * Compiles the behavior for a specific object.
     */
    compile(id) {
        return this.parts.map((part, idx) => part(id, idx)).join("\n");
    }
    /**
     * Compiles the post behavior for a specific object.
     */
    compilePost(id, actions, prefix) {
        for (const priority in this.helpers) {
            if (!actions.hasOwnProperty(priority))
                actions[priority] = [];
            actions[priority].push(...this.helpers[priority]);
        }
        //Only register helpers once.
        this.helpers = {};
        for (const key in this.postParts) {
            if (!actions.hasOwnProperty(key))
                actions[key] = [];
            actions[key].push(prefix);
            actions[key].push(...this.postParts[key].map((part, idx) => part(id, idx)));
        }
    }
    compileDisplay(display) {
        for (const priority in this.displays) {
            if (!display.hasOwnProperty(priority))
                display[priority] = [];
            display[priority].push(...this.displays[priority]);
        }
    }
}
exports.Behavior = Behavior;
//# sourceMappingURL=Behavior.js.map