import { TemplateState } from "./TemplateState";
export declare class SemiMutable {
    private readonly varName;
    private readonly id;
    private val;
    private mutable;
    private inlined;
    private incrementID;
    private variableIDs;
    readonly exportID: number;
    /**
     * Creates a SemiMutable, which stores an immutable value until it is made mutable.
     * Once it is made mutable, it gets converted to a variable, and all mutations can
     * only be done from Desmos.
     * @param varName {string} - is the variable name of this variable.
     * @param id {number} - is the ID of the GameObject that this is attached to.
     * @param val {number} - is the default value.
     * @param state {TemplateState} - is the current state.
     */
    constructor(varName: string, id: number, val: number, state: TemplateState);
    /**
     * Makes a SemiMutable mutable.
     */
    mut(): void;
    /**
     * Returns if this is mutable.
     */
    isMut(): boolean;
    /**
     * Makes a SemiMutable inlined.
     */
    inline(): void;
    /**
     * Returns if this is inlined.
     */
    isInline(): boolean;
    /**
     * Sets the value.
     */
    set(val: number): void;
    /**
     * Gets the value. If supplied, an offset is added to the increment.
     */
    get(offset?: number): string;
    /**
     * Gets the variable name. If supplied, an offset is added to the increment.
     */
    name(noIncrement?: boolean, offset?: number): string;
    /**
     * Increment the value's identifier.
     * @param variable {boolean} - is a value indicating whether this increment should be treated as a variable and not a function.
     */
    increment(variable?: boolean): void;
    /**
     * Decrement the value's identifier.
     */
    decrement(): void;
}
