"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getShortVariableName = exports.getFullVariableName = exports.getBlock = exports.getNumOrString = exports.getAnyAsString = exports.getString = exports.getBoolean = exports.getNum = exports.prefabCheck = exports.behaviorCheck = exports.getSemiMut = exports.objectVarCheck = exports.objectCheck = exports.outerInnerCheck = exports.expressionCheck = exports.outerCheck = exports.ensureState = void 0;
const logimat_1 = require("logimat");
const ensureState = (state) => {
    if (!state.hasOwnProperty("graphgame"))
        state.graphgame = { objects: {}, behaviors: {}, actions: {}, finalActions: [], finalized: false, lastObjectBehaviorId: null, lastObjectBehaviorArgs: null, postActions: [], selects: [], currentObjectId: null, postInit: false, prefabs: [], toIncrement: null, currentBehavior: "", currentPrefab: "", currentObject: 0, nextObjectId: 1, nextVariableId: 1 };
    if (state.graphgame.finalized)
        throw new Error("Do not run any other templates after finalizing!");
};
exports.ensureState = ensureState;
//Context checks
const outerCheck = (context) => {
    if (context !== logimat_1.TemplateContext.OuterDeclaration)
        throw new Error("This can only be ran from outside of any functions.");
};
exports.outerCheck = outerCheck;
const expressionCheck = (context) => {
    if (context !== logimat_1.TemplateContext.Expression)
        throw new Error("This can only be ran from within an expression.");
};
exports.expressionCheck = expressionCheck;
const outerInnerCheck = (context) => {
    if (context === logimat_1.TemplateContext.Expression)
        throw new Error("This cannot be ran from within an expression.");
};
exports.outerInnerCheck = outerInnerCheck;
//GameObject checks
const objectCheck = (state, id) => {
    if (!state.graphgame.objects.hasOwnProperty(id))
        throw new Error("An object with the ID \"" + id + "\" does not exist!");
};
exports.objectCheck = objectCheck;
const objectVarCheck = (state, id, name) => {
    if (!state.graphgame.objects[id].hasOwnProperty(name))
        throw new Error("A variable with the name \"" + name + "\" does not exist!");
};
exports.objectVarCheck = objectVarCheck;
const getSemiMut = (state, id, name) => {
    return state.graphgame.objects[id][name];
};
exports.getSemiMut = getSemiMut;
//Behavior checks
const behaviorCheck = (state, name) => {
    if (!state.graphgame.behaviors.hasOwnProperty(name))
        throw new Error("A behavior with the name \"" + name + "\" does not exist!");
};
exports.behaviorCheck = behaviorCheck;
//Prefab checks
const prefabCheck = (state, name) => {
    if (!state.graphgame.prefabs.hasOwnProperty(name))
        throw new Error("A prefab with the name \"" + name + "\" does not exist!");
};
exports.prefabCheck = prefabCheck;
//Arg checks
const getNum = (args, state, idx, error = null) => {
    if (args.length < idx + 1 || args[idx] == null || typeof (args[idx]) !== "number" || isNaN(args[idx])) {
        if (error)
            throw new Error(error);
        else
            return null;
    }
    return args[idx];
};
exports.getNum = getNum;
const getBoolean = (args, state, idx, error = null) => {
    if (args.length < idx + 1 || args[idx] == null || typeof (args[idx]) !== "boolean") {
        if (error)
            throw new Error(error);
        else
            return false;
    }
    return args[idx];
};
exports.getBoolean = getBoolean;
const getString = (args, state, idx, error = null) => {
    if (args.length < idx + 1 || args[idx] == null || typeof (args[idx]) !== "string") {
        if (error)
            throw new Error(error);
        else
            return null;
    }
    return args[idx];
};
exports.getString = getString;
const getAnyAsString = (args, state, idx, error = null) => {
    if (args.length < idx + 1 || args[idx] == null) {
        if (error)
            throw new Error(error);
        else
            return null;
    }
    return args[idx].toString();
};
exports.getAnyAsString = getAnyAsString;
const getNumOrString = (args, state, idx, error = null) => {
    if (args.length < idx + 1 || args[idx] == null || (typeof (args[idx]) != "string" && typeof (args[idx]) != "number")) {
        if (error)
            throw new Error(error);
        else
            return null;
    }
    return args[idx];
};
exports.getNumOrString = getNumOrString;
const getBlock = (args, state, idx, error = null) => {
    if (args.length < idx + 1 || args[idx] == null || typeof (args[idx]) !== "object" || !args[idx]["block"]) {
        if (error)
            throw new Error(error);
        else
            return null;
    }
    return args[idx]["value"];
};
exports.getBlock = getBlock;
//Dot access helper
const getFullVariableName = (varName, behaviorName) => (varName.startsWith("base.") ? varName.substring(5) : behaviorName + varName).replace(/[._]/g, "");
exports.getFullVariableName = getFullVariableName;
const getShortVariableName = (varName) => varName.split(".").pop();
exports.getShortVariableName = getShortVariableName;
//# sourceMappingURL=util.js.map